/* PWorld - An IRC ProxyScan, m_noscan.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 *
 * Scanning exceptions (on IPs or IRC Server).
 *
 * $Id: m_noscan.c,v 1.3 2001/10/30 11:15:44 mbuna Exp $
 */


#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>
#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif

#include "conf.h"
#include "hashtab.h"
#include "match.h"
#include "m_log.h"

extern pthread_mutex_t mutex_server;
extern scanthread_s *scanthread;
extern struct Server *mServer;
extern htab *mHS;

void
check_noscan_ip(const struct in_addr * client_ip, int *outNoscan)
{
   struct IPrangelist *iprge;
   
   unsigned long ipmin, ipmax, ip;
   
   /* Check if the IP is currently in this range of IPs... */
   ip = ntohl((unsigned long) client_ip->s_addr);
   
   iprge = MyNoScan->ips;
   while(iprge != NULL) {
   
      ipmin = (unsigned long) iprge->start.s_addr;
      ipmax = (unsigned long) iprge->stop.s_addr;
      
      /* We don't recheck each time here if ipmin <= ipmax, assuming load_conf did that well. */
      if ((ipmin <= ip) && (ip <= ipmax)) {
         *outNoscan = true;
         return;
      }
      
      iprge = iprge->next;
   }
   *outNoscan = false;
}

/* Called from any threads */
void
check_noscan_server(unsigned int xserv, int *outNoscan)
{
   *outNoscan = false;   /* Default value */
   
   pthread_mutex_lock(&mutex_server);
   if (hfirst(mHS)) do {
      if(((struct Server *)hstuff(mHS))->myx == xserv) {
         *outNoscan = ((struct Server *)hstuff(mHS))->noscan;
         break;
      }
   } while(hnext(mHS));
   pthread_mutex_unlock(&mutex_server);
}

/* Must be called inside a mutex_server critical zone */
void
refresh_noscan_servers()
{
   struct Server *sptr;
   struct Serverlist *nss;
   
   if (hfirst(mHS)) do {
      sptr = (struct Server *) hstuff(mHS);
      
      sptr->noscan = false; /* default = scan */
      
      for(nss = MyNoScan->residences; nss; nss = nss->next) {
         if(!match(nss->name, sptr->name)) {
            sptr->noscan = true;
            break;
         }
      }
   } while(hnext(mHS));
}
      
/*
   struct Server *sptr;
   struct Serverlist *nss;
   
   sptr = mServer;
   
   while(sptr) {
      
      sptr->noscan = false;   // By default, scan this server's users.
      
      nss = MyNoScan->residences;
      
      while(nss) {
         if(!match(nss->name, sptr->name)) {
            sptr->noscan = true;   // Server name is matching a noscan server -- no scan.
            break;
         }
         nss = nss->next;
      }
      
      sptr = sptr->next;
   }
   */


void
force_refresh_noscan_servers()
{
   pthread_mutex_lock(&mutex_server);
   refresh_noscan_servers();
   pthread_mutex_unlock(&mutex_server);
}

/* m_noscan.c */
