/* PWorld - An IRC ProxyScan, m_share.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 *
 *
 * Methods for handling several proxy scanner on a same IRC network.
 * Shared tasks.
 *
 * $Id: m_share.c,v 1.2 2001/10/20 01:31:03 mbuna Exp $
 */


/*

share {
        name = ProxyScan.US.IRCAide.Org
        successor = ProxyScan2.EU.IRCAide.Org
        server = *.US.IRCAide.Org
}

*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>


#include "conf.h"
#include "hashtab.h"
#include "m_share.h"
#include "match.h"
#include "misc.h"

extern struct Server *mServer;
extern htab *mHS;

void
check_shared_server(unsigned int xserv, int * outNoscan)
{
   *outNoscan = 0;   /* Default value */
   
   if (hfirst(mHS)) do {
      if(((struct Server *)hstuff(mHS))->myx == xserv) {
         *outNoscan = (int)((struct Server *)hstuff(mHS))->shared;
         break;
      }
   } while(hnext(mHS));
}



/* Must be called inside a mutex_server critical zone */
void
refresh_shared_servers()
{
   struct Share_s *p;
   struct Server *sptr;
   struct Serverlist *nss;
   Boolean gotshared;
   
   /* Default, set all servers as not shared */
   if (hfirst(mHS)) do {
      sptr = (struct Server *)hstuff(mHS);
      sptr->shared = false;
   } while(hnext(mHS));
   
   
   /* Loop thru all shared proxyscan */
   for(p = MyShare; p; p = p->next) {
      
      /* We need to know if the shared proxyscan is currently linked.
       * If not, are we its successor? if not, check if the successor is here.
       */
      
      gotshared = false;
      
      /* p->name is already in lowercase (modified in conf.c) */
      if(hfind(mHS, p->name, strlen(p->name) + 1)) {
         gotshared = true;
      }
      
      /*
      sptr = mServer;
      while(sptr) {
         if(!stricmp(sptr->name, p->name)) {
            gotshared = true;
         }
         sptr = sptr->next;
      }
      */
      
      if(!gotshared) {
         
         /* ack! the shared proxyscan is not currently available.
          * check its successor...
          */
         
         gotshared = chk_shared_successor(p);
      }
      
      if(gotshared) {
         
         /* Ok, we found a shared proxyscan which will deal with these servers's clients.
          * If not (or the successor is 'me'), we will scan 'em.
          */
         
         if (hfirst(mHS)) do {
            sptr = (struct Server *)hstuff(mHS);
            
            for(nss = p->servlist; nss; nss = nss->next) {
               
               if(!match(nss->name, sptr->name)) {
                  sptr->shared = true;   /* Server name is matching a shared server. */
                  break;
               }
            }
         } while(hnext(mHS));
   /*
         for(sptr = mServer; sptr; sptr = sptr->next) {
            
            for(nss = p->servlist; nss; nss = nss->next) {
               
               if(!match(nss->name, sptr->name)) {
                  sptr->shared = true;
                  break;
               }
            }
         }          */
      }
   }
}


Boolean
chk_shared(const char *name)
{
   struct Share_s *p;
   
   if(NULL == name)
      return false;
   
   for(p = MyShare; p; p = p->next) {
      if(!stricmp(p->name, name))
         return true;
   }
   return false;
}


/* Check if a shared proxyscanner has a linked successor. If found, return true.
 * If not found, or if the successor is "me", return false.
 */
Boolean
chk_shared_successor(struct Share_s *shared)
{
   //struct Server *sptr;
   
   /* I'm the successor. Return FALSE and we'll scan 'em. */
   if(!stricmp(shared->successor, "me"))
      return false;
      
   else {
   
      /* Check if the successor is linked... */
      
      if(hfind(mHS, shared->successor, strlen(shared->successor) + 1)) {
         return true;
      }
      /*
      for(sptr = mServer; sptr; sptr = sptr->next) {
      
         if(!stricmp(sptr->name, shared->successor))
            return true;
      }
      */
   }
   
   /* If we are here, we didn't found the successor, so we'll scan 'em. */
   return false;
}


/* m_share.c */
