/* Pxys - An IRC ProxyScan, misc.c
 * 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * Miscellaneous functions
 *
 * $Id: misc.c,v 1.1.1.1 2001/10/14 22:10:53 mbuna Exp $
 */



#include "config.h"
#include "misc.h"

#include <time.h>
#include <string.h>
#include <ctype.h>


#if defined(HAVE_WCTYPE_H) && defined(HAVE_ISWBLANK)
#include <wctype.h>
#endif



void
stripline(char *ptr)
{
   for(; ptr && *ptr; ++ptr) {
      if(('\r' == *ptr) || ('\n' == *ptr)) {
         *ptr = '\0';
         return;
      }
   }
}

void
stripserv(char *fullname)
{
   char *ptr;

   if((ptr = strchr(fullname, '.'))) {
      *ptr = '\0';
   }
}

char *
strlwr(char *io)
{
   char *ptr = io;
   for(; ptr && *ptr; ++ptr) {
      *ptr = tolower(*ptr);
   }
   return io;
}


#ifdef HAVE_STRCASECMP

int
stricmp(const char *s1, const char *s2)
{
   return strcasecmp(s1, s2);
}

#else  /* HAVE_STRCASECMP */

int
stricmp(const char *s1, const char *s2)
{
   register int c;
   while((c = tolower(*s1)) == tolower(*s2)) {
      if(c == 0)
         return 0;
      s1++;
      s2++;
   }
   if(c < tolower(*s2))
      return -1;
   return 1;
}

#endif  /* HAVE_STRCASECMP */


int iswblank(int c)      { return ((c == (int)L' ') || (c == (int)L'\t')); }

#if HAVE_ISWBLANK

void
trim_left(char **s)
{
   while(s && *s && iswblank(**s)) (*s)++;
}


void
trim_right(char *s)
{
   int i;
   if(s == NULL)
      return;
   for(i = strlen(s); i > 0; i--) {
      if(iswblank(s[i - 1])) {
         s[i - 1] = '\0';
      } else {
         return;
      }
   }
}


#else	/* HAVE_ISWBLANK */

void
trim_left(char **s)
{
   while(s && *s && (isspace(**s) || (**s == '\t'))) (*s)++;
}


void
trim_right(char *s)
{
   int i;
   if(s == NULL)
      return;
   for(i = strlen(s); i > 0; i--) {
      if(isspace(s[i - 1]) || (s[i - 1] == '\t')) {
         s[i - 1] = '\0';
      } else {
         return;
      }
   }
}

#endif	/* HAVE_ISWBLANK */

