/* Pxys - An IRC ProxyScan, ms_myclient.c
 * 
 * Copyright (C) 2001 Stephane Thiell <mbuna@undernet.org>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * 
 * ProxyScan "virtual" client.
 *
 * $Id: ms_myclient.c,v 1.11 2001/11/21 15:43:23 mbuna Exp $
 */
 

#include "config.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "misc.h"
#include "conf.h"
#include "debug.h"
#include "hashtab.h"

#include "ms_myclient.h"
#include "ms_msgout.h"
#include "m_array.h"
#include "m_log.h"
#include "m_opercmds.h"
#include "m_stats.h"
#include "m_structs.h"
#include "m_shutdown.h"
#include "ms_auth.h"
#include "ms_globals.h"
#include "ms_netutils.h"


extern struct Client * mClient;
extern struct Server * mServer;
extern const char MyClientXXX[];
extern htab *mHS;
extern htab *mHC;

htab *cmdstab = NULL;

/* Proc Ptr */
typedef void (*opercmdsPP)(struct Client *, const char *, const char*);

struct opercmds_s {
    const char *cmd;
    opercmdsPP func;
};

typedef struct opercmds_s opercmds_s;

/* Commands list, lowercases only */
static opercmds_s opercmds_list[] = {
	{ "help", opercmds_help },
	{ "inv", opercmds_inv },
	{ "status", opercmds_status },
	{ "links", opercmds_links },
	{ "info", opercmds_info },
	{ "lastcomm", opercmds_lastcomm },
	{ "lastlog", opercmds_lastlog },
	{ "grem", opercmds_grem },
	{ "recheck", opercmds_recheck },
	{ "stats", opercmds_stats },
	{ "wannounce", opercmds_wannounce },
	{ "showopr", opercmds_showopr },
	{ "listshowopr", opercmds_listshowopr }
};

void
myclient_init()
{
   int i;
   
   cmdstab = jenkins_hcreate(2);

#define NUMOPERCMDS sizeof(opercmds_list)/sizeof(opercmds_s)
   
/* Register oper commands */
   for (i = 0; i < NUMOPERCMDS; i++)
      hadd(cmdstab, opercmds_list[i].cmd, strlen(opercmds_list[i].cmd),
            (void*)opercmds_list[i].func);
   
}


void
myclient_destroy()
{
   if (cmdstab) {
      jenkins_hdestroy(cmdstab);
      cmdstab = NULL;
   }
}


void
myclient_handle_privmsg(char *nn, char *cmd, char *arg1)
{
   struct Client *cptr;
#ifndef N2K_SUPPORT
   char charYXX[4];
#endif
   
   if (!(nn && cmd)) /* BLAW#@! */
      return;
   
   if(*cmd == ':')
      ++cmd;
   
   strlwr(cmd);   /* only deal with commands in lowercase */
   
#ifdef N2K_SUPPORT
   cptr = array_findclient_from_numeric(nn);
#else
   if (*nn == ':') ++nn;
   cptr = array_findclient_from_nick(nn);
#endif

   if(NULL == cptr)
      /* Should never happen :( */
      log_error("myclient_handle_privmsg: userid %s not found!", nn);
      
   else {
      /* *Must* be an oper for *all* opercmds */
      if (cptr->flags & FLAG_OPER) {
#ifndef N2K_SUPPORT
         /* If NOT n2k, build numnick (ok that sucks) */
         inttobase64(charYXX, cptr->nserv, 1);
         inttobase64(charYXX + 1, cptr->nnick, 2);
         nn = charYXX;
#endif
         /* Call the appropriate procedure... */
         if (hfind(cmdstab, cmd, strlen(cmd)))
            (*(opercmdsPP)hstuff(cmdstab))(cptr, nn, arg1);
         else
            client_sendto(nn, "Sorry?");
      }
   }
}

void
myclient_handle_join(const char *channel, char *nn)
{
   struct Client *cptr;
   
   cptr = array_findclient_from_numeric(nn);
   if(NULL == cptr) {
      Debug((DBLMAX, "on_join: user %s not found!", nn));
   } else {
      if(cptr->flags & FLAG_OPER) {
         /* Auto-Op for opers */
         sendto_uplink("%s%s M %s +o %s", MyYY, MyClientXXX, channel, nn);
      }/*
      else {
         sendto_uplink("%s%s K %s %s :Not *'d", MyYY, MyClientXXX, channel, nn);
      }*/
   }
}


/* ms_myclient.c */
